# Eudonet CRM API Client

This module provides a PHP client to interact with the Eudonet API.

## Available queries

### Authentication

```php
# Create an authentication query object.
$query = \Drupal::service('eudonet')->getAuthenticationQuery();

# Short-hand authentication.
\Drupal::service('eudonet')->authenticate();

# Get authentication token.
$token = \Drupal::service('eudonet')->getAuthToken();
```

### Catalog

```php
Drupal::service('eudonet')->catalog(267);
```

### Meta information

```php
Drupal::service('eudonet')->metaInfos(<tablist>);
```

### Search
```php
Drupal::service('eudonet')->search(200);
```

## Concepts

2 plugin types are used by this module:

* The EudonetQuery plug-in type is responsible for the http request to the eudonet API.
* The EudonetQueryResult plug-in type is used to abstract (facilitate handling) the response.

## Examples

### Search with conditions

```php
$query = $this->eudonet->search(200);
$query->addFields(['last_name', 'first_name', 'gender']);

$query->condition('gender', 'Masculin');
$query->condition('last_name', NULL, 'NOT EMPTY');

$or = $query->orGroup();
$or->condition('last_name', 'XE', '^');
$or->condition('last_name', 'A.M.B.', '^');
$query->conditionGroup($or);

$result = $query->execute();
/** @var \Drupal\eudonet\EudonetSearchResultItem $item */
foreach ($result as $item) {
  $last_name = $item->last_name?->Value;
  $file_id = $item->id();
}
```

### Search catalog

```php
$result = $this->eudonet->catalog(267)->execute();
foreach ($result as $item) {
  $label = $item['Label'];
  $id = $item['Id'];
}
```
