<?php

namespace Drupal\eudonet\Plugin\EudonetQuery;

use Drupal\Core\Http\ClientFactory;
use Drupal\eudonet\Eudonet;
use Drupal\eudonet\Plugin\EudonetQueryBase;
use Drupal\eudonet\Plugin\EudonetQueryResultManager;

/**
 * Class CUDQuery.
 *
 * @package Drupal\eudonet\Plugin\EudonetQuery
 *
 * @EudonetQuery(
 *   id = "eudonet_cud_query",
 *   label = @Translation("Create/Update/Delete"),
 *   path = "CUD/",
 *   method = "POST",
 *   authentication = TRUE,
 * )
 */
class CUDQuery extends EudonetQueryBase {

  protected array $fields;

  protected ?array $imageValue;

  /**
   * {@inheritdoc}
   */
  public function getMethod() {
    if (str_starts_with($this->configuration['additional_path'], 'Delete')) {
      return 'DELETE';
    }

    return parent::getMethod();
  }

  /**
   * {@inheritdoc}
   */
  public function getPath() {
    return parent::getPath() . $this->configuration['additional_path'];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    if (!empty($this->fields)) {
      $build['Fields'] = $this->fields;
    }
    elseif (!empty($this->imageValue)) {
      $build = $this->imageValue;
    }
    return $build;
  }

  /**
   * Set a value for a specific field.
   *
   * This method MUST not be used from a Update Image query.
   * Use setImageValue() instead.
   *
   * @param int|string $field_name
   *   The field_name to update.
   * @param null|int|string|bool $value
   *   The value to set.
   *
   * @return $this
   */
  public function setValue(int|string $field_name, null|string|int|bool $value): static {
    $this->ensureMapping();
    $this->fields[] = [
      'DescId' => $this->getFieldId($field_name),
      'Value' => $value,
    ];
    return $this;
  }

  /**
   * Set multiple values(field_name => $value)
   *
   * @param array $fields
   *   An associative array of values, keyed by field name.
   *
   * @return $this
   *
   * @see setValue()
   */
  public function setValues(array $fields): static {
    foreach ($fields as $field_name => $value) {
      $this->setValue($field_name, $value);
    }
    return $this;
  }

  /**
   * Set an image value for a specific field.
   *
   * @param int|string $field_name
   *   The field name to update.
   * @param mixed $value
   *   The value to set.
   *
   * @return $this
   */
  public function setImageValue(int|string $field_name, null|int|string|bool $value): static {
    $this->ensureMapping();
    $this->imageValue = [
      'DescId' => $this->mapping[$field_name] ?? $field_name,
      'Value' => $value,
    ];
    return $this;
  }

}
