<?php

namespace Drupal\eudonet\Plugin\EudonetQueryResult;

use Drupal\eudonet\Eudonet;
use Drupal\eudonet\EudonetSearchResultItem;
use Drupal\eudonet\Plugin\EudonetQueryResultBase;
use Drupal\eudonet\Traits\EudonetMappingTrait;
use Drupal\eudonet\Traits\EudonetQueryResultPagerTrait;

/**
 * Class SearchQueryResult.
 *
 * @package Drupal\eudonet\Plugin\EudonetQueryResult
 *
 * @EudonetQueryResult(
 *   id = "eudonet_search_query_result",
 *   label = @Translation("Search query result")
 * )
 */
class SearchQueryResult extends EudonetQueryResultBase implements \Iterator, \Countable {

  use EudonetMappingTrait;
  use EudonetQueryResultPagerTrait;

  /**
   * The current table index.
   *
   * @var int
   */
  private int $position;

  /**
   * The tables.
   *
   * @var \Drupal\eudonet\EudonetSearchResultItem[]
   */
  private array $items;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, string $plugin_id, $plugin_definition, Eudonet $eudonet) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $eudonet);
    $this->setMapping($configuration['mapping'] ?? 'default');
    $this->ensureMapping();
    $this->items = $this->prepareItems();
  }

  /**
   * Get the table name of the search.
   */
  public function label() {
    return $this->response['ResultMetaData']['Tables'][0]['Label'] ?? '';
  }

  /**
   * Get all items.
   *
   * @return array
   *   Result items.
   */
  public function getItems(): array {
    return $this->items;
  }

  /**
   * Prepare all result items as objects.
   *
   * @return array
   *   Result items.
   */
  protected function prepareItems(): array {
    // Use result class from mapping, if defined.
    $class = $this->mapping['_class'] ?? EudonetSearchResultItem::class;

    if (!class_exists($class)) {
      throw new \InvalidArgumentException("Undefined class: $class");
    }

    $baseClass = EudonetSearchResultItem::class;
    if (
      $class !== $baseClass &&
      !is_subclass_of($class, $baseClass)
    ) {
      throw new \InvalidArgumentException("$class must extend $baseClass");
    }

    $result = [];
    foreach ($this->response['ResultData']['Rows'] ?? [] as $row) {
      // @todo Key items by their FileID.
      $result[] = new $class($row, $this->configuration['mapping'] ?? 'default');
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
  public function current() {
    return $this->items[$this->position];
  }

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
  public function next() {
    ++$this->position;
  }

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
  public function key() {
    return $this->position;
  }

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
  public function valid() {
    return isset($this->items[$this->position]);
  }

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
  public function rewind() {
    $this->position = 0;
  }

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
  public function count() {
    return count($this->items);
  }

}
