<?php

namespace Drupal\eudonet\Traits;

use Drupal\eudonet\Exception\MappingException;

/**
 * Trait EudonetMappingTrait.
 *
 * @todo Consider refactoring.
 *   - Create a EudonetMapping object that represents each eudonet mapping.
 *   - Rename this trait to EudonetMappingAwareTrait.
 *   - The trait should have a ::setMapping(string $id).
 *   - The trait should have a ::getMapping(): EudonetMapping.
 *   - To get DescId for fields, always use ::getMapping()->getFieldId().
 *   - To get model class, use ::getMapping()->getModelClass().
 *
 * @package Drupal\eudonet\Traits
 */
trait EudonetMappingTrait {

  /**
   * The current mapping Plugin ID.
   *
   * @var string
   */
  protected string $mappingId;

  /**
   * The current mapping definition.
   *
   * @var array
   */
  protected array $mapping;

  /**
   * Set the mapping for a query or a query result.
   *
   * @param string $mapping_id
   *   The mapping plugin ID.
   *
   * @return $this
   */
  public function setMapping(string $mapping_id): static {
    $this->mappingId = $mapping_id;
    $this->ensureMapping();

    return $this;
  }

  /**
   * Make sure to get the plugin definition.
   */
  protected function ensureMapping(): void {
    if (!empty($this->mapping)) {
      return;
    }

    if (isset($this->eudonetMapping)) {
      $this->mapping = $this->eudonetMapping->getDefinition($this->mappingId);
      return;
    }

    $this->mapping = \Drupal::service('plugin.manager.eudonet_mapping')
      ->getDefinition($this->mappingId);
  }

  /**
   * Use mapping data to convert a field name to Field ID (DescId).
   *
   * @param string|int $field_name
   *   A field name or a field ID.
   *
   * @return int
   *   Field ID, i.e. DescId.
   */
  public function getFieldId(string|int $field_name): int {
    $this->ensureMapping();

    if (is_numeric($field_name)) {
      if (in_array($field_name, $this->mapping)) {
        return (int) $field_name;
      }

      throw new \InvalidArgumentException("Attempt to set illegal property: $field_name");
    }

    if (!isset($this->mapping[$field_name]) || empty($this->mapping[$field_name])) {
      throw new MappingException("Unmapped field: $field_name");
    }

    if (!is_numeric($this->mapping[$field_name])) {
      throw new MappingException("Non-numeric value for key: $field_name");
    }

    return $this->mapping[$field_name];
  }

  public function getMappedFieldNames(): array {
    $this->ensureMapping();
    $result = $this->mapping;

    unset($result['_class']);
    unset($result['provider']);

    return array_keys($result);
  }

}
