<?php

namespace Drupal\eudonet\Traits;

use Drupal\eudonet\Eudonet;

/**
 * Trait EudonetQueryOrderByTrait.
 *
 * @package Drupal\eudonet\Traits
 */
trait EudonetQueryOrderByTrait {

  protected array $orderBy = [
    'OrderBy' => [],
  ];

  /**
   * Order the query by a field name.
   *
   * @param int|string $field_name
   *   A mapped field name or a DescId.
   * @param string $order
   *   The sort order. See Eudonet::SORT_ORDERS.
   *
   * return $this
   */
  public function orderBy(string|int $field_name, string $order = 'ASC'): static {
    if (!isset(Eudonet::SORT_ORDERS[$order])) {
      throw new \InvalidArgumentException("Unrecognized order: $order");
    }

    if (method_exists($this, 'ensureMapping')) {
      $this->ensureMapping();
    }

    $field = $field_name;
    if (isset($this->mapping) && !empty($this->mapping[$field_name])) {
      $field = $this->mapping[$field_name];
    }

    $this->orderBy['OrderBy'][] = [
      'DescId' => $field,
      'Order' => Eudonet::SORT_ORDERS[$order],
    ];

    return $this;
  }

}
